/*
 * Copyright 2010, 2011, 2012, 2013 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "StateManager.h"
#include "RaceState.h"
#include "Path.h"
#include "MainMenu.h"
#include "Version.h"
#include "SDL_image.h"
#include <cstdlib>
#include <ctime>
#include <cstring>
#include <cstdio>

// IMG_Init/IMG_Quit was introduced in SDL_image 1.2.8
#if SDL_IMAGE_MAJOR_VERSION < 1 || (SDL_IMAGE_MAJOR_VERSION == 1 && (SDL_IMAGE_MINOR_VERSION < 2 || (SDL_IMAGE_MINOR_VERSION == 2 && SDL_IMAGE_PATCHLEVEL < 8)))
#define IMG_Init(x)
#define IMG_Quit()
#endif

void handleProgramArguments(int argc, char *argv[]);

int main(int argc, char *argv[])
{
	if (argc > 1)
	{
		handleProgramArguments(argc, argv);
		return 0;
	}
	
	// seed the random number
	std::srand(std::time(0));

	// Initialize SDL
	SDL_Init(SDL_INIT_TIMER|SDL_INIT_VIDEO|SDL_INIT_JOYSTICK);
	IMG_Init(IMG_INIT_PNG);
	SDL_EnableUNICODE(true);
	SDL_WM_SetCaption("Brum Brum Rally", 0);
	if (SDL_Surface* icon = IMG_Load((dataPath() + "/images/icon.png").c_str()))
	{
		SDL_WM_SetIcon(icon, 0);
		SDL_FreeSurface(icon);
	}

	// run the state manager
	StateManager& stateManager = StateManager::getInstance();
	stateManager.enter(new MainMenu());
	stateManager.run();
	
	// cleans up SDL
	stateManager.cleanUp();
	IMG_Quit();
	SDL_Quit();
	return 0;
}

void handleProgramArguments(int argc, char *argv[])
{
	for (int i = 1; i < argc; ++i)
	{
		if (std::strcmp(argv[i], "-?") == 0 ||
		    std::strcmp(argv[i], "-h") == 0 ||
		    std::strcmp(argv[i], "--help") == 0)
		{
			std::printf("You can use the following options:\n");
			std::printf("  -c --config-path   Show location where configuration files are stored.\n");
			std::printf("  -d --data-path     Show location where read-only data files are stored.\n");
			std::printf("  -h --help          Show this help message.\n");
			std::printf("  -r --reset         Reset all settings to default values.\n");
			std::printf("  -v --version       Show Brum Brum Rally version information.\n");
		}
		else if (std::strcmp(argv[i], "-c") == 0 ||
		    std::strcmp(argv[i], "--config-path") == 0)
		{
			std::printf("%s\n", configPath().c_str());
		}
		else if (std::strcmp(argv[i], "-d") == 0 ||
		    std::strcmp(argv[i], "--data-path") == 0)
		{
			std::printf("%s\n", dataPath().c_str());
		}
		else if (std::strcmp(argv[i], "-r") == 0 ||
		    std::strcmp(argv[i], "--reset") == 0)
		{
			Path settingsPath(configPath() + "settings");
			std::remove(settingsPath.c_str());
			if (!settingsPath.exist())
			{
				std::printf("The settings has been reset!\n");
			}
			else
			{
				std::printf("Unable to reset settings!\n");
			}
		}
		else if (std::strcmp(argv[i], "-v") == 0 ||
		    std::strcmp(argv[i], "--version") == 0)
		{
			std::printf("Brum Brum Rally %s\n", BBR_VERSION);
		}
		else
		{
			std::printf("Unknown option: %s\n", argv[i]);
		}
	}
}
